@@ -131,6 +131,7 @@ struct saver_preferences {
   Bool use_mit_saver_extension;
   Bool use_sgi_saver_extension;
   Bool use_proc_interrupts;
+  Bool use_xinput_extension;
 
   Bool getviewport_full_of_lies_p; /* XFree86 bug #421 */
 
@@ -188,8 +189,20 @@ struct saver_info {
 # ifdef HAVE_RANDR
   int randr_event_number;
   int randr_error_number;
+  Bool using_randr_extension;
 # endif
 
+  Bool using_xinput_extension;     /* Note that `p->use_*' is the *request*, */
+                                   /* and `si->using_*' is the *reality*.    */
+#ifdef HAVE_XINPUT
+  int xinput_ext_event_number;     /* may not be used */
+  int xinput_ext_error_number;
+  int xinput_DeviceButtonPress;    /* Extension device event codes.          */
+  int xinput_DeviceButtonRelease;  /* Assigned by server at runtime          */
+  int xinput_DeviceMotionNotify;
+  struct xinput_dev_info *xinput_devices;
+  int num_xinput_devices;
+# endif
 
   /* =======================================================================
      blanking
